function Get-MWRegKey([Parameter(Mandatory=$true)][ValidateSet('SC','OM')][string]$MWComponent)
{
    $DictMWComponents = New-Object "System.Collections.Generic.Dictionary``2[System.String,System.String]"
    $DictMWComponents.Add('SC', 'Service Center')
    $DictMWComponents.Add('OM', 'Onsite Manager')
    
    if ([System.IntPtr]::Size -eq 4) {
        [string]$RegBasePath = "HKLM:\SOFTWARE\Level Platforms\Managed Workplace\{0}\Install"
    }
    elseif ([System.IntPtr]::Size -eq 8) {
        [string]$RegBasePath = "HKLM:\SOFTWARE\Wow6432Node\Level Platforms\Managed Workplace\{0}\Install"   
    }

    $RegPath = $([string]::Format($RegBasePath, $DictMWComponents[$MWComponent]))
    
    if (Test-Path $RegPath)
    {
        $host.UI.WriteLine("Detected Registry key.")
        $Script:DBType = "EXPRESS"
        return Get-ItemProperty $RegPath 
    }

    throw [System.NullReferenceException]::new("Registry key not found")
}

Function Invoke-SQLDatabaseNonQuery # create a function we'll use to query the database
{
    Param ([string]$ConnectionString,$query)
 
    $connection = New-Object System.Data.SqlClient.SqlConnection # create db connection object
    $connection.ConnectionString = $ConnectionString
    $connection.Open() #open connection
    $command = $connection.CreateCommand()
    $command.CommandText = $Query # run query
    $command.Parameters.Add("@utcdate", [System.Data.SqlDbType]::DateTime2)
    $command.Parameters["@utcdate"].Value = [datetime](Get-date).ToUniversalTime()
    $result = $command.ExecuteNonQuery()
    $connection.Close() # close the connection
    $connection.Dispose() 
}

Function Invoke-SQLCEDatabaseNonQuery # create a function we'll use to query the DM database
{
    Param ([string]$ConnectionString,$query)
 
    $connection = New-Object "System.Data.SqlServerCe.SqlCeConnection" $ConnectionString
    $command = New-Object "System.Data.SqlServerCe.SqlCeCommand"
    $command.CommandType = [System.Data.CommandType]"Text"
    $command.Connection = $connection
    $connection.Open()
    $command.CommandText = $query
    $command.Parameters.Add("@utcdate", [System.Data.SqlDbType]::DateTime)
    $command.Parameters["@utcdate"].Value = [datetime](Get-date).ToUniversalTime()
    $result = $command.ExecuteNonQuery()
    $command.Dispose()
    $connection.Close(); 
    $connection.Dispose | Out-Null
}

Function LoadSqlServerCE
{
    $host.UI.WriteLine("Loading SQL CE Library.")
    $binpath = Join-Path $Script:regkey.Path "Bin\System.Data.SqlServerCe.dll";
    [Reflection.Assembly]::LoadFile($binpath) | Out-Null
}

$Script:DBType = $null
$Script:regkey = $null

try
{
    $Script:regkey = Get-MWRegKey -MWComponent OM
}
catch [Exception]
{
    $host.UI.WriteErrorLine($_.Message)
    [Environment]::Exit(5)
}

if (![string]::IsNullOrEmpty($Script:regkey.IsOML))
{
    if ([bool]$Script:regkey.IsOML)
    {
        $host.UI.WriteLine("Device Manager detected.")
        $Script:DBType = "SQLCE"
    }
}

$MWVersion = New-Object System.Version($Script:regkey.MWProductVersion)
if ($MWVersion.Major -gt 11 -and $MWVersion.Minor -ne 0)
{
    $msg = [string]::Format("Managed Workplace {0} is not supported by this script", $MWVersion.ToString() )
    $host.UI.WriteErrorLine($msg)
    [environment]::Exit(5)
}

$queryText = "
INSERT INTO [AvastAgentAction]
           ([ActionID]
           ,[DeviceID]
           ,[ServiceID]
           ,[Status]
           ,[Command]
           ,[CmdParameters]
           ,[CmdCreated]
           ,[CmdStarted]
           ,[CmdCompleted]
           ,[ResultCode]
           ,[ResultMessage]
           ,[Retries]
           ,[ParentActionID])
SELECT 1000000000 + S.DeviceID,
       S.DeviceID,
	   1,
	   3,
	   'InstallAvastAgentService',
	   '{`"RequireMandatoryRebootAfterInstall`":false,`"CheckLastBootUpTimeFromDeviceAssets`":false,`"NextCheckForRebootPending`":null,`"CheckedAvmForPolicyAssignment`":false,`"SettingsId`":`"' + S.InstallationToken + '`",`"ManagementUrl`":null}',
	   DATEADD(MINUTE, -20, @utcdate),
	   DATEADD(MINUTE, -17, @utcdate),
	   DATEADD(MINUTE, -11, @utcdate),
	   2011,
	   '',
	   1,
	   NULL
FROM AvastAgentService S
WHERE S.ServiceID = 1
AND   S.StateForServiceCenter = 3
AND   S.ReasonCode = 2012
"
$queryText2 = "
UPDATE [AvastAgentService]
   SET [StateForServiceCenter] = 2
      ,[ReasonCode] = 2011
WHERE ServiceID = 1
AND   StateForServiceCenter = 3
AND   ReasonCode = 2012
"

try
{
    If ($Script:DBType -eq "EXPRESS")
    {
        $host.UI.WriteLine("Updating SQL Express Database")
        $builder = New-Object System.Data.SqlClient.SqlConnectionStringBuilder($Script:regkey.ConnectionString)
        $builder["Initial Catalog"] = "MWData"

        Invoke-SQLDatabaseNonQuery -ConnectionString $builder.ToString() -query $queryText
		Invoke-SQLDatabaseNonQuery -ConnectionString $builder.ToString() -query $queryText2
    }
    elseif ($Script:DBType -eq "SQLCE") 
    {
        LoadSqlServerCE

        $host.UI.WriteLine("Updating SQLCE Database")
        $MWDataSDFPath = Join-Path $Script:regkey.Path "Bin\MWData.sdf"
        $SqlCeConnectionString = [string]::Format("Data Source='{0}';", $MWDataSDFPath)
        Invoke-SQLCEDatabaseNonQuery -ConnectionString $SqlCeConnectionString -query $queryText
		Invoke-SQLCEDatabaseNonQuery -ConnectionString $SqlCeConnectionString -query $queryText2
    }    
}
catch [Exception]
{
    $msg = [string]::Format("Error occurred while executing database statement: {0}",[environment]::NewLine + $_ )
    $host.UI.WriteErrorLine($msg)
    #[environment]::Exit(5)
}

$host.UI.WriteLine("Script Completed successfully")